from django.db import models
from django import forms

class SoftDeleteManager(models.Manager):

    def get_queryset(self):
        return super().get_queryset().filter(is_deleted=False)

class SoftDeleteModel(models.Model):

    is_deleted = models.BooleanField(default=False)
    objects = SoftDeleteManager()
    all_objects = models.Manager()

    def soft_delete(self):
        self.is_deleted = True
        self.save(update_fields=["is_deleted"])

    def restore(self):
        self.is_deleted = False
        self.save(update_fields=["is_deleted"])

    class Meta:
        abstract = True

class ExamCategory(SoftDeleteModel):
    exam_category_name = models.CharField(max_length=255, default='NULL')
    image = models.ImageField(upload_to='learn/exam_category_images/', default='NULL')
    status = models.CharField(max_length=255, default='NULL')
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

class Exam_Master(SoftDeleteModel):
    exam_category=models.ForeignKey(ExamCategory, on_delete=models.CASCADE, default='NULL')
    exam_name = models.CharField(max_length=255, default='NULL')
    image = models.ImageField(upload_to='learn/exam_master_images/', default='NULL')
    banner = models.ImageField(upload_to='learn/exam_master_banner_images/', default='NULL')
    marks_per_question = models.FloatField()
    negative_marks_per_question = models.FloatField()
    status = models.CharField(max_length=255, default='NULL')
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now_add=True)


class Sections(SoftDeleteModel):
    section_name = models.CharField(max_length=255, default='NULL')
    display_order = models.CharField(max_length=255, default='NULL')
    status = models.CharField(max_length=255, default='NULL')
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)  

class Sub_Sections(SoftDeleteModel):
    # section = models.CharField(max_length=255)
    section = models.ForeignKey(Sections, on_delete=models.CASCADE, default='NULL')
    sub_section_name = models.CharField(max_length=255, default='NULL')
    display_order = models.CharField(max_length=255, default='NULL')
    status = models.CharField(max_length=255, default='NULL')
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True) 

class Topics(SoftDeleteModel):
    section = models.ForeignKey(Sections, on_delete=models.CASCADE, default='NULL')
    sub_section = models.ForeignKey(Sub_Sections, on_delete=models.CASCADE, default='NULL')
    topic_name = models.CharField(max_length=255, default='NULL')
    display_order = models.CharField(max_length=255, default='NULL')
    status = models.CharField(max_length=255, default='NULL')
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True) 

class Question_Bank(SoftDeleteModel):
    section = models.ForeignKey(Sections, on_delete=models.CASCADE, default='NULL')
    sub_section = models.ForeignKey(Sub_Sections, on_delete=models.CASCADE, default='NULL')
    topic = models.ForeignKey(Topics, on_delete=models.CASCADE, default='NULL')
    question_type = models.CharField(max_length=255 , default='NULL')
    difficulty_level = models.CharField(max_length=255 , default='NULL')
    time_to_speed = models.CharField(max_length=255 , default='NULL')
    question = models.CharField(max_length=255 , default='NULL')
    explanation = models.CharField(max_length=255 , default='NULL')
    status = models.CharField(max_length=255 , default='NULL')
    question_file = models.ImageField(upload_to='learn/question/' , default='NULL')
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True) 

class Option(SoftDeleteModel):
    question = models.ForeignKey(Question_Bank, on_delete=models.CASCADE)
    option = models.CharField(max_length=100)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    is_deleted = models.BooleanField(default=False)


class Mock_Test(SoftDeleteModel):
    section = models.ForeignKey(Sections, on_delete=models.CASCADE, default='NULL')
    sub_section = models.ForeignKey(Sub_Sections, on_delete=models.CASCADE, default='NULL')
    topic = models.ForeignKey(Topics, on_delete=models.CASCADE, default='NULL')
    examcategory = models.ForeignKey(ExamCategory, on_delete=models.CASCADE, default='NULL')
    mock_title = models.CharField(max_length=255 , default='NULL')
    mock_type = models.CharField(max_length=255 , default='NULL')
    mock_level = models.CharField(max_length=255 , default='NULL')
    mock_duration = models.CharField(max_length=255 , default='NULL')
    pass_per = models.CharField(max_length=255 , default='NULL')
    negative_marks = models.CharField(max_length=255 , default='NULL')
    instruction_page = models.CharField(max_length=255 , default='NULL')
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True) 

class Faculty(SoftDeleteModel):
    faculty_name = models.CharField(max_length=255,default="Null")
    email = models.CharField(max_length=255,default="Null")
    password = models.CharField(max_length=255,default="Null")
    contact = models.CharField(max_length=255,default="Null")
    experience = models.CharField(max_length=255,default="Null")
    date_of_birth = models.DateField(max_length=255,default="Null")
    status = models.CharField(max_length=255,default="Null")
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True) 

class Batch_Management(SoftDeleteModel):
    batch_name = models.CharField(max_length=255 ,default="Null")
    couse_name = models.CharField(max_length=255,default="Null")
    status = models.CharField(max_length=255,default="Null")
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)  

class Students(SoftDeleteModel):
    batch = models.ForeignKey(Batch_Management, on_delete=models.CASCADE, default='NULL')
    first_name = models.CharField(max_length=255 ,default="Null")
    last_name = models.CharField(max_length=255,default="Null")
    email = models.CharField(max_length=255,default="Null")
    password = models.CharField(max_length=255,default="Null")
    mobile = models.CharField(max_length=255,default="Null")
    address = models.CharField(max_length=255,default="Null")
    landmark = models.CharField(max_length=255,default="Null")
    city = models.CharField(max_length=255,default="Null")
    state = models.CharField(max_length=255,default="Null")
    pincode = models.CharField(max_length=255,default="Null")
    remark = models.CharField(max_length=255,default="Null")
    status = models.CharField(max_length=255,default="Null")
    is_lead = models.IntegerField(default=0)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)  




class Exam_Subjects(SoftDeleteModel):
    section = models.ForeignKey(Sections, on_delete=models.CASCADE, default='NULL')
    exam= models.ForeignKey(Exam_Master, on_delete=models.CASCADE, default='NULL')
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now_add=True)

class Live_Lecture(SoftDeleteModel):
    title = models.CharField(max_length=255,default="Null")
    batch = models.ForeignKey(Batch_Management, on_delete=models.CASCADE)
    subject = models.CharField(max_length=255,default="Null")
    topic = models.ForeignKey(Topics, on_delete=models.CASCADE)
    image = models.ImageField(upload_to='learn/live_lecture_images/',default="Null")
    faculty = models.ForeignKey(Faculty, on_delete=models.CASCADE)
    lacture_id = models.CharField(max_length=255,default="Null")
    lacture_date = models.DateField(max_length=255,default="Null")
    start_time = models.TimeField(max_length=255,default="Null")
    end_time = models.TimeField(max_length=255,default="Null")
    password = models.CharField(max_length=255,default="Null")
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now_add=True) 


class Blog_category(SoftDeleteModel):
    category_name = models.CharField(max_length=255,default="Null")
    blog_category_image = models.ImageField(upload_to='learn/blog_category_images/', default='NULL')
    meta_title = models.CharField(max_length=255,default="Null")
    meta_discription = models.CharField(max_length=255,default="Null")
    slug = models.CharField(max_length=255,default="Null")
    status = models.CharField(max_length=255,default="Null")
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True) 

class Blogs(SoftDeleteModel):
    category = models.ForeignKey(Blog_category, on_delete=models.CASCADE, default='NULL')
    blog_title = models.CharField(max_length=255,default="Null")
    blog_image = models.ImageField(upload_to='learn/blog_image/', default='NULL')
    blog_discription = models.CharField(max_length=255,default="Null")
    blog_status = models.CharField(max_length=255,default="Null")
    meta_title = models.CharField(max_length=255,default="Null")
    meta_discription = models.CharField(max_length=255,default="Null")
    slug = models.CharField(max_length=255,default="Null")
    status = models.CharField(max_length=255,default="Null")
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)  

class Banners(SoftDeleteModel):
    banner_image = models.ImageField(upload_to='learn/banner_images/', default='NULL')
    section = models.ForeignKey(Sections, on_delete=models.CASCADE, default='NULL')
    status = models.CharField(max_length=255,default="Null")
    redirect_url= models.CharField(max_length=255,default="Null")
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now_add=True)

class Course(SoftDeleteModel):
    course_name = models.CharField(max_length=500)
    image = models.ImageField(upload_to='course_images/')
    exam_category = models.ForeignKey(ExamCategory, on_delete=models.CASCADE)
    exam_master = models.ForeignKey(Exam_Master, on_delete=models.CASCADE)
    duration = models.IntegerField()
    description = models.TextField()
    area_price = models.DecimalField(max_digits=8, decimal_places=2)
    sectional_price = models.DecimalField(max_digits=8, decimal_places=2)
    minimock_price = models.DecimalField(max_digits=8, decimal_places=2)
    fullmock_price = models.DecimalField(max_digits=8, decimal_places=2)
    lectures_price = models.DecimalField(max_digits=8, decimal_places=2)
    full_price = models.DecimalField(max_digits=8, decimal_places=2)
    status = models.CharField(max_length=20)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)


class MediaModule(SoftDeleteModel):
    image = models.ImageField(upload_to='learn/media/', default='NULL')
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)